#include <States/GameState/GameState.h>
#include <States/SelectState/SelectState.h>

void GameState::event_handling() {
    sf::Event event{};
    while (m_window.pollEvent(event)) {
        if (event.type == sf::Event::Closed) {
            m_state_manager.set_next_state(std::make_unique<SelectState>(m_state_manager, config::SELECT_LEVEL_VIDEO_MODE, config::SELECT_LEVEL_TITLE));
        }
    }
}

void GameState::update() {
    if (m_context_manager.get_current_context().state != GameContext::INGAME)
        return;
    for (const auto& obj : m_context_manager.get_current_context().dynamic_objects)
        obj->action();
}

void GameState::render() {
    switch (m_context_manager.get_current_context().state) {
        case GameContext::INGAME:
            m_window.clear(config::GAME_COLOR_BACKGROUND_INGAME);
            break;
        case GameContext::WIN:
            m_window.clear(config::GAME_COLOR_BACKGROUND_WIN);
            break;
        case GameContext::LOST:
            m_window.clear(config::GAME_COLOR_BACKGROUND_LOST);
    }
    m_window.clear(config::GAME_COLOR_BACKGROUND_INGAME);
    m_maze.draw_into(m_window);
    for (const auto& obj : m_context_manager.get_current_context().static_objects)
        obj->draw_into(m_window);
    for (const auto& obj : m_context_manager.get_current_context().dynamic_objects)
        obj->draw_into(m_window);
    m_context_manager.get_current_context().pacman.draw_into(m_window);
    m_window.display();
}

bool GameState::do_step() {
    event_handling();
    update();
    render();
    return true;
}
