#include <States/SelectState/SelectState.h>
#include <States/ChangeStateCommand.h>
#include <States/GameState/GameBuilder/GameBuilderDirector.h>

Menu::Menu(IStateManager& state_manager) {
    const float pos_left = (static_cast<float>(config::SELECT_LEVEL_VIDEO_MODE.width) - config::BUTTON_SIZE.x) / 2;
    const float pos_diff = (static_cast<float>(config::SELECT_LEVEL_VIDEO_MODE.height) - config::BUTTON_SIZE.y * 5) / 10;
    const float pos_top = (static_cast<float>(config::SELECT_LEVEL_VIDEO_MODE.height) - config::BUTTON_SIZE.y*5 - pos_diff*4) / 2;

    auto easy_level_command = std::make_unique<GameCommand>(
        state_manager,
        std::make_unique<GameBuilderDirector>(
            std::make_unique<SimpleBuilder>(config::GAME_VIDEO_MODE.width, config::GAME_VIDEO_MODE.height, config::ROOM_SIZE, config::GAME_COLOR_ROOM),
            config::GAME_VIDEO_MODE,
            config::EASY_GAME_TITLE,
            config::EASY_GAME_ENEMY_RATIO));

    auto medium_level_command = std::make_unique<GameCommand>(
        state_manager,
        std::make_unique<GameBuilderDirector>(
            std::make_unique<SimpleBuilder>(config::GAME_VIDEO_MODE.width, config::GAME_VIDEO_MODE.height, config::ROOM_SIZE, config::GAME_COLOR_ROOM),
            config::GAME_VIDEO_MODE,
            config::MEDIUM_GAME_TITLE,
            config::MEDIUM_GAME_ENEMY_RATIO));

    auto hard_level_command = std::make_unique<GameCommand>(
        state_manager,
        std::make_unique<GameBuilderDirector>(
            std::make_unique<ComplexBuilder>(config::GAME_VIDEO_MODE.width, config::GAME_VIDEO_MODE.height, config::ROOM_SIZE, config::GAME_COLOR_ROOM),
            config::GAME_VIDEO_MODE,
            config::HARD_GAME_TITLE,
            config::HARD_GAME_ENEMY_RATIO));

    auto extreme_level_command = std::make_unique<GameCommand>(
        state_manager,
        std::make_unique<GameBuilderDirector>(
            std::make_unique<ComplexRandomBuilder>(config::GAME_VIDEO_MODE.width, config::GAME_VIDEO_MODE.height, config::ROOM_SIZE, config::GAME_COLOR_ROOM),
            config::GAME_VIDEO_MODE,
            config::HARD_GAME_TITLE,
            config::HARD_GAME_ENEMY_RATIO));

    m_buttons[0].set(sf::Vector2f{pos_left, pos_top}, config::BUTTON_SIZE, config::BUTTON_TEXT_EASY,
        config::BUTTON_FONT_SIZE, std::move(easy_level_command));
    m_buttons[1].set(sf::Vector2f{pos_left, pos_top + config::BUTTON_SIZE.y + pos_diff}, config::BUTTON_SIZE,
     config::BUTTON_TEXT_MEDIUM, config::BUTTON_FONT_SIZE, std::move(medium_level_command));
    m_buttons[2].set(sf::Vector2f{pos_left, pos_top + 2*(config::BUTTON_SIZE.y + pos_diff)}, config::BUTTON_SIZE,
     config::BUTTON_TEXT_HARD, config::BUTTON_FONT_SIZE, std::move(hard_level_command));
    m_buttons[3].set(sf::Vector2f{pos_left, pos_top + 3*(config::BUTTON_SIZE.y + pos_diff)}, config::BUTTON_SIZE,
    config::BUTTON_TEXT_EXTREME, config::BUTTON_FONT_SIZE, std::move(extreme_level_command));
    m_buttons[4].set(sf::Vector2f{pos_left, pos_top + 4*(config::BUTTON_SIZE.y + pos_diff)}, config::BUTTON_SIZE,
     config::BUTTON_TEXT_EXIT, config::BUTTON_FONT_SIZE, std::make_unique<ExitCommand>(state_manager));
}

void Menu::process_mouse(const sf::Vector2f pos, const bool is_pressed) {
    for (auto& button : m_buttons) {
        if (!button.is_position_in(pos)) {
            button.unselect();
            continue;
        }
        if (!is_pressed) {
            button.select();
            continue;
        }
        button.push();
    }
}

void Menu::draw_into(sf::RenderWindow& window) const {
    for (auto& button : m_buttons) {
        button.draw_into(window);
    }
}

void SelectState::event_handling() {
    sf::Event event{};
    while (m_window.pollEvent(event)) {
        if (event.type == sf::Event::Closed) {
            m_state_manager.set_next_state(std::make_unique<ExitState>(m_state_manager));
            //m_window.close(); ///< @todo убрать вероятно
        }
        if (event.type == sf::Event::Resized) {
            sf::View view = m_window.getView();
            view.setSize(static_cast<float>(event.size.width), static_cast<float>(event.size.height));
            m_window.setView(view);
        }
    }
}

void SelectState::update() {
    m_menu.process_mouse(m_window.mapPixelToCoords(sf::Mouse::getPosition(m_window)),
        sf::Mouse::isButtonPressed(sf::Mouse::Left));
}

void SelectState::render() {
    m_window.clear(config::SELECT_LEVEL_BACKGROUND_COLOR);
    m_menu.draw_into(m_window);
    m_window.display();
}

bool SelectState::do_step() {
    event_handling();
    update();
    render();
    return true;
}